<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */
namespace FiloBlu\Esb\Observer;
use \Magento\Sales\Api\Data\OrderInterface;
use \FiloBlu\Esb\Helper\Data;
use \Magento\Framework\Event\Observer;
use \Magento\Framework\Event\ObserverInterface;

/**
 * Observer on invoice register
 */
 
 class InvoiceRegister implements ObserverInterface{
	
    /**
     * @var \FiloBlu\Esb\Model\ScheduleFactory
     */  	
    protected $_scheduleFactory;
    
	/**
     * @var \Psr\Log\LoggerInterface $logger
     */  
    protected $_logger;

	/**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $_scopeConfig;

	/**
     * @var \FiloBlu\Esb\Helper\Data
     */  
    protected $_helper;

        /** 
     * @var \Magento\Framework\Serialize\SerializerInterface 
     */
    protected $_serializer;
    
	/**
     * @param \FiloBlu\Esb\Model\ScheduleFactory $scheduleFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \FiloBlu\Esb\Helper\Data $helper
     */  
	public function __construct(
            \FiloBlu\Esb\Model\ScheduleFactory $scheduleFactory, 
            \Psr\Log\LoggerInterface $logger,
            \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
            \FiloBlu\Esb\Helper\Data $helper,
            \Magento\Framework\Serialize\SerializerInterface $serializer    
	){
            $this->_scheduleFactory=$scheduleFactory;
            $this->_logger = $logger;
            $this->_scopeConfig = $scopeConfig;
            $this->_helper = $helper;
            $this->_serializer = $serializer;
	}

    /**
     * Send data to ESB schedule after invoice register. 
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return \Magento\Sales\Api\Data\OrderInterface 
     */
    public function execute(Observer $observer){    
        $schedule = $this->_scheduleFactory->create();
        $order = $observer->getEvent()->getOrder();
        $events=$this->_serializer->unserialize($this->_scopeConfig->getValue(
            'esb/endpoints/events', 
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $order->getStoreId()
        ));  
        if($events!=null && count($events)>0){
            foreach($events as $eventData){
                if($eventData["event_type"]!="invoice_register"){continue;}
                $endpoint=$this->_helper->getEndpointUrl($eventData["event_endpoint"]);
                $priority=($eventData["priority"])?$eventData["priority"]:0;   
                $schedule->setData(
                    array(
                        "object_repository_interface" => "\Magento\Sales\Api\OrderRepositoryInterface",
                        "object_id" => $order->getId(),
                        "endpoint" => $endpoint,
                        "event" => "sales_order_invoice_register",
                        "sendmessage_class" => "\FiloBlu\Esb\Cron\SendMessage",
                        "status" => "pending",
                        "priority" => $priority,
                        "executable_after" => new \Zend_Db_Expr('NOW()')
                    )
                );
                $schedule->save();        
            }
        }
        return $order;
    }
}